/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.importer;

import org.bioimageanalysis.icy.extension.plugin.abstract_.Plugin;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.io.SequenceFileImporter;
import org.bioimageanalysis.icy.model.image.IcyBufferedImage;
import org.bioimageanalysis.icy.model.roi.ROI;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.tools.io.IOBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarInteger;
import plugins.adufour.vars.lang.VarROIArray;
import plugins.adufour.vars.util.VarException;

import java.awt.*;

/**
 * Block to returns an image from the given parameters and opened SequenceFileImporter.
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Get Image from SequenceFileImporter")
@IcyPluginIcon(path = "/SequenceBlocks_icon.png")
public class SequenceFileImporterGetImage extends Plugin implements IOBlock {
    final protected Var<PositionedSequenceFileImporter> importer;
    final protected VarInteger series;
    final protected VarInteger resolution;
    final protected VarROIArray region;
    final protected VarInteger zIndex;
    final protected VarInteger tIndex;
    final protected VarInteger cIndex;

    final protected Var<IcyBufferedImage> image;

    public SequenceFileImporterGetImage() {
        super();

        importer = new Var<>("Importer", PositionedSequenceFileImporter.class);
        series = new VarInteger("Series", -1);
        resolution = new VarInteger("Resolution (0=full, 1=1/2, ..)", 0);
        region = new VarROIArray("XY region (ROI)", null);
        zIndex = new VarInteger("Z (slice) index", -1);
        tIndex = new VarInteger("T (frame) index", -1);
        cIndex = new VarInteger("C (channel) index", -1);

        image = new Var<>("Image", IcyBufferedImage.class);
    }

    @Override
    public void run() {
        final PositionedSequenceFileImporter pi = importer.getValue();

        if (pi == null)
            throw new VarException(importer, "Importer is null !");

        final SequenceFileImporter imp = pi.importer;

        if (imp.getOpened() == null)
            throw new VarException(importer, "Importer is not opened !");

        final int res = resolution.getValue();
        final ROI[] rois = region.getValue();

        int s = series.getValue();
        int z = zIndex.getValue();
        int t = tIndex.getValue();
        int c = cIndex.getValue();
        Rectangle rect = ((rois != null) && (rois.length > 0)) ? rois[0].getBounds5D().toRectangle2D().getBounds()
                : null;

        // undefined values ? use internal position if any defined
        if ((s == -1) && (pi.s != -1))
            s = pi.s;
        if ((z == -1) && (pi.z != -1))
            z = pi.z;
        if ((t == -1) && (pi.t != -1))
            t = pi.t;
        if ((c == -1) && (pi.c != -1))
            c = pi.c;
        if ((rect == null) && (pi.xyRegion != null))
            rect = pi.xyRegion;

        // still undefined ? --> set default value for series
        if (s == -1)
            s = 0;

        try {
            image.setValue(imp.getImage(s, res, rect, z, t, c));
        }
        catch (final Exception e) {
            throw new VarException(importer, e.getMessage());
        }
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("importer", importer);
        inputMap.add("series", series);
        inputMap.add("resolution", resolution);
        inputMap.add("region", region);
        inputMap.add("zIndex", zIndex);
        inputMap.add("tIndex", tIndex);
        inputMap.add("cIndex", cIndex);
    }

    @Override
    public void declareOutput(final @NotNull VarList outputMap) {
        outputMap.add("image", image);
    }
}
