/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.importer;

import org.bioimageanalysis.icy.common.reflect.ClassUtil;
import org.bioimageanalysis.icy.extension.plugin.abstract_.Plugin;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.io.Loader;
import org.bioimageanalysis.icy.io.SequenceFileImporter;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.tools.io.IOBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.gui.model.ValueSelectionModel;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarInteger;
import plugins.adufour.vars.lang.VarMutable;
import plugins.adufour.vars.lang.VarString;
import plugins.adufour.vars.util.VarException;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Block to open a file with a given SequenceFileImporter.
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Open File w/ SequenceFileImporter")
@IcyPluginIcon(path = "/SequenceBlocks_icon.png")
public class SequenceFileImporterOpen extends Plugin implements IOBlock {
    final static String ID_AUTO = "Automatic";

    final protected VarString importerName;
    final protected Var<PositionedSequenceFileImporter> importer;
    final protected VarMutable file;
    final protected VarInteger flag;

    // internal
    final protected Map<String, SequenceFileImporter> importersMap;

    public SequenceFileImporterOpen() {
        super();

        final List<String> importersName = new ArrayList<>();
        final List<SequenceFileImporter> importers = Loader.getSequenceFileImporters();

        importerName = new VarString("Select importer", "", 1 + importers.size());
        // build importer map
        importersMap = new HashMap<>();

        // automatic selection
        importersName.add(ID_AUTO);
        importersMap.put(ID_AUTO, null);
        for (final SequenceFileImporter importer : importers) {
            String className = ClassUtil.getBaseClassName(importer.getClass().getName());
            final int ind = className.lastIndexOf('.');

            // just get final class name without package name
            if (ind > -1)
                className = className.substring(ind + 1);

            importersName.add(className);
            importersMap.put(className, importer);
        }

        // initialize importer selector field
        importerName.setDefaultEditorModel(new ValueSelectionModel<>(importersName.toArray(new String[0]), importersName.getFirst(), false));

        importer = new Var<>("Importer", PositionedSequenceFileImporter.class);
        file = new VarMutable("File", null) {
            @Override
            public boolean isAssignableFrom(final Var source) {
                return (String.class == source.getType()) || (File.class == source.getType());
            }
        };
        flag = new VarInteger("Flag", 0);
    }

    @Override
    public void run() {
        final String path;
        final Object obj = file.getValue();

        if (obj != null) {
            if (obj instanceof String)
                path = (String) obj;
            else
                path = ((File) obj).getAbsolutePath();
        }
        else
            throw new VarException(file, "File is null !");

        // get selected importer
        SequenceFileImporter imp = importersMap.get(importerName.getValue());

        // automatic selection from input file ?
        if (imp == null)
            imp = Loader.getSequenceFileImporter(path, true);

        // cannot get importer ?
        if (imp == null)
            throw new VarException(importer, "Cannot find an importer for file: '" + path + "' !");

        // set importer
        importer.setValue(new PositionedSequenceFileImporter(imp));

        try {
            // try to open the importer
            imp.open(path, flag.getValue());
        }
        catch (final Exception e) {
            throw new VarException(importer, e.getMessage());
        }
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("name", importerName);
        inputMap.add("file", file);
        inputMap.add("flag", flag);
    }

    @Override
    public void declareOutput(final @NotNull VarList outputMap) {
        outputMap.add("importer", importer);
    }
}
