/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.infos;

import ome.xml.meta.OMEXMLMetadata;
import org.bioimageanalysis.icy.common.math.UnitUtil;
import org.bioimageanalysis.icy.extension.plugin.abstract_.Plugin;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.model.sequence.Sequence;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.*;
import plugins.adufour.vars.util.VarException;

import java.awt.*;

/**
 * Block to return metadata information from a Sequence object.
 *
 * @author Stephane Dallongeville
 * @see ReadMetadata
 */
@IcyPluginName("Get Metadata")
@IcyPluginIcon(path = "/SequenceBlocks_icon.png")
public class GetMetaData extends Plugin implements SequenceBlock {
    final protected VarSequence varSequence = new VarSequence("Sequence", null);
    final protected Var<OMEXMLMetadata> metaData = new Var<>("Metadata", OMEXMLMetadata.class);
    final protected VarString name = new VarString("Name", "");
    final protected VarInteger seriesIndex = new VarInteger("Series index", 0);
    final protected VarDouble pxSizeX = new VarDouble("Pixel Size X (" + UnitUtil.MICRO_STRING + "m)", 1d);
    final protected VarDouble pxSizeY = new VarDouble("Pixel Size Y (" + UnitUtil.MICRO_STRING + "m)", 1d);
    final protected VarDouble pxSizeZ = new VarDouble("Pixel Size Z (" + UnitUtil.MICRO_STRING + "m)", 1d);
    final protected VarDouble timeIntT = new VarDouble("Time interval T (s)", 100d);
    final protected VarDouble positionX = new VarDouble("Position X (" + UnitUtil.MICRO_STRING + "m)", 0d);
    final protected VarDouble positionY = new VarDouble("Position Y (" + UnitUtil.MICRO_STRING + "m)", 0d);
    final protected VarDouble positionZ = new VarDouble("Position Z (" + UnitUtil.MICRO_STRING + "m)", 0d);
    final protected VarInteger originResolution = new VarInteger("Resolution", 0);
    final protected Var<Rectangle> originRegion = new Var<>("Region", new Rectangle());
    final protected VarInteger originZMin = new VarInteger("Z min", -1);
    final protected VarInteger originZMax = new VarInteger("Z max", -1);
    final protected VarInteger originTMin = new VarInteger("T min", -1);
    final protected VarInteger originTMax = new VarInteger("T max", -1);
    final protected VarInteger originChannel = new VarInteger("Channel", -1);

    @Override
    public void run() {
        final Sequence s = varSequence.getValue();

        if (s == null)
            throw new VarException(varSequence, "Sequence is null");

        metaData.setValue(s.getOMEXMLMetadata());
        name.setValue(s.getName());
        seriesIndex.setValue(s.getSeries());
        pxSizeX.setValue(s.getPixelSizeX());
        pxSizeY.setValue(s.getPixelSizeY());
        pxSizeZ.setValue(s.getPixelSizeZ());
        timeIntT.setValue(s.getTimeInterval());
        positionX.setValue(s.getPositionX());
        positionY.setValue(s.getPositionY());
        positionZ.setValue(s.getPositionZ());
        originResolution.setValue(s.getOriginResolution());
        originRegion.setValue(s.getOriginXYRegion());
        originZMin.setValue(s.getOriginZMin());
        originZMax.setValue(s.getOriginZMax());
        originTMin.setValue(s.getOriginTMin());
        originTMax.setValue(s.getOriginTMax());
        originChannel.setValue(s.getOriginChannel());
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("sequence", varSequence);
    }

    @Override
    public void declareOutput(final @NotNull VarList outputMap) {
        outputMap.add("metadata", metaData);
        outputMap.add("name", name);
        outputMap.add("seriesIndex", seriesIndex);
        outputMap.add("Pixel Size X (mm)", pxSizeX);
        outputMap.add("Pixel Size Y (mm)", pxSizeY);
        outputMap.add("Pixel Size Z (mm)", pxSizeZ);
        outputMap.add("Time interval T (ms)", timeIntT);
        outputMap.add("positionx", positionX);
        outputMap.add("positiony", positionY);
        outputMap.add("positionz", positionZ);
        outputMap.add("originResolution", originResolution);
        outputMap.add("originRegion", originRegion);
        outputMap.add("originZMin", originZMin);
        outputMap.add("originZMax", originZMax);
        outputMap.add("originTMin", originTMin);
        outputMap.add("originTMax", originTMax);
        outputMap.add("originChannel", originChannel);
    }
}
