/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.loop;

import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.io.SequenceFileImporter;
import plugins.adufour.blocks.lang.Batch;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarInteger;
import plugins.adufour.vars.util.VarException;
import plugins.tprovoost.sequenceblocks.importer.PositionedSequenceFileImporter;

import java.util.List;

/**
 * Base abstract class for {@link SequenceFileImporter} batch loop.
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("SequenceFileImporter Batch")
@IcyPluginIcon(path = "/SequenceBlocks_icon.png")
public abstract class SequenceFileImporterBatch extends Batch implements SequenceBlock {
    // important to not initialize and create them in getBatchSource() and getBatchElement()
    protected Var<PositionedSequenceFileImporter> positionedImporter;
    protected Var<PositionedSequenceFileImporter> element;
    protected VarInteger series;

    // internal
    protected int limit;

    @Override
    public Var<PositionedSequenceFileImporter> getBatchSource() {
        // initialize variable if needed
        if (positionedImporter == null)
            positionedImporter = new Var<>("Importer",
                    PositionedSequenceFileImporter.class);

        return positionedImporter;
    }

    @Override
    public Var<PositionedSequenceFileImporter> getBatchElement() {
        // initialize element if needed
        if (element == null)
            element = new Var<>("Loop importer", PositionedSequenceFileImporter.class);

        return element;
    }

    @Override
    public void initializeLoop() {
        final PositionedSequenceFileImporter value = positionedImporter.getValue();

        if (value == null)
            throw new VarException(positionedImporter, "Input importer is null !");

        final SequenceFileImporter imp = value.importer;

        if (imp.getOpened() == null)
            throw new VarException(positionedImporter, "Importer is not opened !");

        // create new positioned importer for element initialization
        final PositionedSequenceFileImporter pi = new PositionedSequenceFileImporter(value);

        final int s = series.getValue();

        // defined series ? --> set series position
        if (s != -1)
            pi.s = s;

        // init element with a copy of current positioned importer (and eventually set serie position)
        element.setValue(pi);
    }

    @Override
    public void declareInput(final VarList inputMap) {
        super.declareInput(inputMap);

        // lazy creation
        if (series == null)
            series = new VarInteger("Series", -1);

        inputMap.add("series", series);
    }

    @Override
    public void declareLoopVariables(final List<Var<?>> loopVariables) {
        // need to be called after
        super.declareLoopVariables(loopVariables);

        // lazy creation
        if (series == null)
            series = new VarInteger("Series", -1);

        loopVariables.add(series);
    }
}
