/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.loop;

import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.io.SequenceFileImporter;
import org.bioimageanalysis.icy.model.image.ImageUtil;
import org.bioimageanalysis.icy.model.sequence.MetaDataUtil;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarInteger;
import plugins.adufour.vars.util.VarException;
import plugins.tprovoost.sequenceblocks.importer.PositionedSequenceFileImporter;

import java.awt.*;
import java.util.List;

/**
 * Simple batch loop to iterate over all XY tile from the specified opened {@link SequenceFileImporter} object.
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("SequenceFileImporter Tile Batch")
@IcyPluginIcon(path = "/SequenceBlocks_icon.png")
public class SequenceFileImporterTileBatch extends SequenceFileImporterBatch {
    // important to not initialize here (even with null) and create them in getBatchSource() and getBatchElement()
    protected VarInteger tileW;
    protected VarInteger tileH;

    protected List<Rectangle> tiles;

    @Override
    public void initializeLoop() {
        super.initializeLoop();

        // can take position from element which should have been initialized here
        final PositionedSequenceFileImporter pi = element.getValue();

        // can iterate over XY dimension ?
        if (pi.xyRegion == null) {
            final SequenceFileImporter imp = pi.importer;
            final int s = (pi.s == -1) ? 0 : pi.s;
            final int sizeX;
            final int sizeY;
            int tw;
            int th;

            try {
                sizeX = MetaDataUtil.getSizeX(pi.getMetadata(), s);
                sizeY = MetaDataUtil.getSizeY(pi.getMetadata(), s);

                tw = tileW.getValue();
                th = tileH.getValue();
                // auto ? --> get it from importer
                if (tw == -1)
                    tw = imp.getTileWidth(s);
                // auto ? --> get it from importer
                if (th == -1)
                    th = imp.getTileHeight(s);
            }
            catch (final Exception e) {
                throw new VarException(element, "Error while initializing SequenceFileImporter tile batch: " + e.getMessage());
            }

            // any size supported ? --> use 1024 by default
            if (tw == -1)
                tw = 1024;
                // tile loading not supported ? --> use full width
            else if (tw == 0)
                tw = sizeX;
            // any size supported ? --> use 1024 by default
            if (th == -1)
                th = 1024;
                // tile loading not supported ? --> use full height
            else if (th == 0)
                th = sizeY;

            // get tiles
            tiles = ImageUtil.getTileList(sizeX, sizeY, tw, th);
            // set limit
            limit = tiles.size();
        }
        else
            limit = 1;
    }

    @Override
    public void declareInput(final VarList inputMap) {
        super.declareInput(inputMap);

        // lazy creation
        if (tileW == null)
            tileW = new VarInteger("Tile width (-1 = auto)", -1);
        if (tileH == null)
            tileH = new VarInteger("Tile height (-1 = auto)", -1);

        inputMap.add("tileWidht", tileW);
        inputMap.add("tileHeight", tileH);
    }

    @Override
    public void declareLoopVariables(final List<Var<?>> loopVariables) {
        // need to be called after
        super.declareLoopVariables(loopVariables);

        // lazy creation
        if (tileW == null)
            tileW = new VarInteger("Tile width (-1 = auto)", -1);
        if (tileH == null)
            tileH = new VarInteger("Tile height (-1 = auto)", -1);

        loopVariables.add(tileW);
        loopVariables.add(tileH);
    }

    @Override
    public void beforeIteration() {
        // set current position XY region
        if (positionedImporter.getValue().xyRegion == null) {
            element.getValue().xyRegion = tiles.get(getIterationCounter().getValue());
            // force element changed event so loop get correctly executed
            element.valueChanged(element, element.getValue(), element.getValue());
        }
    }

    @Override
    public boolean isStopConditionReached() {
        return getIterationCounter().getValue() >= limit;
    }
}
